@@ -1,17 +1,48 @@
 # -*- coding: utf-8 -*-
-# ==============================================================================
+#
+#  Copyright (c) 2012--2014, Nico Schlömer, <nico.schloemer@gmail.com>
+#  All rights reserved.
+#
+#  This file is part of VoroPy.
+#
+#  Redistribution and use in source and binary forms, with or without
+#  modification, are permitted provided that the following conditions are met:
+#
+#  1. Redistributions of source code must retain the above copyright notice,
+#  this list of conditions and the following disclaimer.
+#
+#  2. Redistributions in binary form must reproduce the above copyright notice,
+#  this list of conditions and the following disclaimer in the documentation
+#  and/or other materials provided with the distribution.
+#
+#  3. Neither the name of the copyright holder nor the names of its
+#  contributors may be used to endorse or promote products derived from this
+#  software without specific prior written permission.
+#
+#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+#  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+#  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+#  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+#  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+#  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+#  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+#  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+#  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+#  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+#  POSSIBILITY OF SUCH DAMAGE.
+#
 __all__ = ['mesh2d']
 
 import numpy as np
 import warnings
 from base import _base_mesh
-# ==============================================================================
+
+
 class mesh2d(_base_mesh):
-    '''Class for handling two-dimensional triangular meshes.
+    '''Class for handling triangular meshes.
 
     .. inheritance-diagram:: mesh2d
     '''
-    # --------------------------------------------------------------------------
     def __init__(self, nodes, cells=None):
         '''Initialization.
         '''
@@ -37,7 +68,7 @@ class mesh2d(_base_mesh):
         self.cell_circumcenters = None
         self.control_volumes = None
         return
-    # --------------------------------------------------------------------------
+
     def create_cell_volumes(self):
         '''Computes the area of all triangles in the mesh.
         '''
@@ -47,13 +78,16 @@ class mesh2d(_base_mesh):
             # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
             # Shoelace formula.
             node0, node1, node2 = self.node_coords[cell['nodes']]
-            self.cell_volumes[cell_id] = 0.5 * abs( node0[0] * node1[1] - node0[1] * node1[0]
-                                                  + node1[0] * node2[1] - node1[1] * node2[0]
-                                                  + node2[0] * node0[1] - node2[1] * node0[0])
+            self.cell_volumes[cell_id] = \
+                0.5 * abs(node0[0] * node1[1] - node0[1] * node1[0]
+                          + node1[0] * node2[1] - node1[1] * node2[0]
+                          + node2[0] * node0[1] - node2[1] * node0[0]
+                          )
             # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
             #edge0 = node0 - node1
             #edge1 = node1 - node2
-            #self.cell_volumes[cell_id] = 0.5 * np.linalg.norm( np.cross( edge0, edge1 ) )
+            #self.cell_volumes[cell_id] = \
+            #    0.5 * np.linalg.norm(np.cross(edge0, edge1))
             # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
             ## Append a third component.
             #from vtk import vtkTriangle
@@ -62,21 +96,22 @@ class mesh2d(_base_mesh):
                #abs(vtkTriangle.TriangleArea(x[0], x[1], x[2]))
             # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
         return
-    # --------------------------------------------------------------------------
-    def compute_cell_circumcenters( self ):
+
+    def compute_cell_circumcenters(self):
         '''Computes the center of the circumsphere of each cell.
         '''
         from vtk import vtkTriangle
         num_cells = len(self.cells['nodes'])
-        self.cell_circumcenters = np.empty(num_cells, dtype=np.dtype((float, 2)))
+        self.cell_circumcenters = np.empty(num_cells,
+                                           dtype=np.dtype((float, 2))
+                                           )
         for cell_id, cell in enumerate(self.cells):
             x = self.node_coords[cell['nodes']]
             vtkTriangle.Circumcircle(x[0], x[1], x[2],
                                      self.cell_circumcenters[cell_id])
-
         return
-    # --------------------------------------------------------------------------
-    def create_adjacent_entities( self ):
+
+    def create_adjacent_entities(self):
         '''Setup edge-node and edge-cell relations.
         '''
         if self.edges is not None:
@@ -110,16 +145,17 @@ class mesh2d(_base_mesh):
             # We're treating simplices so loop over all combinations of
             # local nodes.
             # Make sure cellNodes are sorted.
-            self.cells['nodes'][cell_id] = np.sort(self.cells['nodes'][cell_id])
+            self.cells['nodes'][cell_id] = \
+                np.sort(self.cells['nodes'][cell_id])
             for k in xrange(len(cell['nodes'])):
                 # Remove the k-th element. This makes sure that the k-th
                 # edge is opposite of the k-th node. Useful later in
                 # in construction of edge (face) normals.
                 indices = tuple(cell['nodes'][:k]) \
-                        + tuple(cell['nodes'][k+1:])
+                    + tuple(cell['nodes'][k+1:])
                 if indices in registered_edges:
                     edge_gid = registered_edges[indices]
-                    self.edges[edge_gid]['cells'].append( cell_id )
+                    self.edges[edge_gid]['cells'].append(cell_id)
                     self.cells[cell_id]['edges'][k] = edge_gid
                 else:
                     # add edge
@@ -129,17 +165,15 @@ class mesh2d(_base_mesh):
                     # http://projects.scipy.org/numpy/ticket/2068
                     self.edges['nodes'][new_edge_gid] = indices
                     # edge['cells'] is also always ordered.
-                    self.edges['cells'][new_edge_gid].append( cell_id )
+                    self.edges['cells'][new_edge_gid].append(cell_id)
                     self.cells['edges'][cell_id][k] = new_edge_gid
                     registered_edges[indices] = new_edge_gid
                     new_edge_gid += 1
-
         # trim edges
         self.edges = self.edges[:new_edge_gid]
-
         return
-    # --------------------------------------------------------------------------
-    def refine( self ):
+
+    def refine(self):
         '''Canonically refine a mesh by inserting nodes at all edge midpoints
         and make four triangular elements where there was one.
         This is a very crude refinement; don't use for actual applications.
@@ -150,7 +184,6 @@ class mesh2d(_base_mesh):
         num_nodes = len(self.node_coords)
         num_new_nodes = len(self.edges)
 
-        new_nodes = np.empty(num_new_nodes, dtype=np.dtype((float, 2)))
         self.node_coords.resize(num_nodes+num_new_nodes, 2, refcheck=False)
         # Set starting index for new nodes.
         new_node_gid = num_nodes
@@ -178,8 +211,8 @@ class mesh2d(_base_mesh):
             # Divide edges.
             local_edge_midpoint_gids = np.empty(3, dtype=int)
             local_edge_newedges = np.empty(3, dtype=np.dtype((int, 2)))
-            local_neighbor_midpoints = [ [], [], [] ]
-            local_neighbor_newedges = [ [], [], [] ]
+            local_neighbor_midpoints = [[], [], []]
+            local_neighbor_newedges = [[], [], []]
             for k, edge_gid in enumerate(cell['edges']):
                 edgenodes_gids = self.edges['nodes'][edge_gid]
                 if is_edge_divided[edge_gid]:
@@ -191,8 +224,9 @@ class mesh2d(_base_mesh):
                 else:
                     # Create new node at the edge midpoint.
                     self.node_coords[new_node_gid] = \
-                        0.5 * (self.node_coords[edgenodes_gids[0]] \
-                              +self.node_coords[edgenodes_gids[1]])
+                        0.5 * (self.node_coords[edgenodes_gids[0]]
+                               + self.node_coords[edgenodes_gids[1]]
+                               )
                     local_edge_midpoint_gids[k] = new_node_gid
                     new_node_gid += 1
                     edge_midpoint_gids[edge_gid] = \
@@ -200,10 +234,12 @@ class mesh2d(_base_mesh):
 
                     # Divide edge into two.
                     new_edges_nodes[new_edge_gid] = \
-                        np.array([edgenodes_gids[0], local_edge_midpoint_gids[k]])
+                        np.array([edgenodes_gids[0],
+                                  local_edge_midpoint_gids[k]])
                     new_edge_gid += 1
                     new_edges_nodes[new_edge_gid] = \
-                        np.array([local_edge_midpoint_gids[k], edgenodes_gids[1]])
+                        np.array([local_edge_midpoint_gids[k],
+                                  edgenodes_gids[1]])
                     new_edge_gid += 1
 
                     local_edge_newedges[k] = [new_edge_gid-2, new_edge_gid-1]
@@ -213,17 +249,18 @@ class mesh2d(_base_mesh):
                     is_edge_divided[edge_gid] = True
                 # Keep a record of the new neighbors of the old nodes.
                 # Get local node IDs.
-                edgenodes_lids = [np.nonzero(cell['nodes'] == edgenodes_gids[0])[0][0],
-                                  np.nonzero(cell['nodes'] == edgenodes_gids[1])[0][0]
-                                  ]
+                edgenodes_lids = \
+                    [np.nonzero(cell['nodes'] == edgenodes_gids[0])[0][0],
+                     np.nonzero(cell['nodes'] == edgenodes_gids[1])[0][0]
+                     ]
                 local_neighbor_midpoints[edgenodes_lids[0]] \
-                    .append( local_edge_midpoint_gids[k] )
+                    .append(local_edge_midpoint_gids[k])
                 local_neighbor_midpoints[edgenodes_lids[1]]\
-                    .append( local_edge_midpoint_gids[k] )
+                    .append(local_edge_midpoint_gids[k])
                 local_neighbor_newedges[edgenodes_lids[0]] \
-                    .append( local_edge_newedges[k][0] )
+                    .append(local_edge_newedges[k][0])
                 local_neighbor_newedges[edgenodes_lids[1]] \
-                    .append( local_edge_newedges[k][1] )
+                    .append(local_edge_newedges[k][1])
 
             new_edge_opposite_of_local_node = np.empty(3, dtype=int)
             # New edges: Connect the three midpoints.
@@ -248,22 +285,20 @@ class mesh2d(_base_mesh):
                               local_neighbor_newedges[k][0],
                               local_neighbor_newedges[k][1]])
                 new_cell_gid += 1
-
         # Override edges.
         num_edges = len(new_edges_nodes)
         self.edges = np.empty(num_edges, dtype=np.dtype([('nodes', (int, 2))]))
         self.edges['nodes'] = new_edges_nodes
-
         # Override cells.
         num_cells = len(new_cells_nodes)
         self.cells = np.empty(num_cells,
-                              dtype=np.dtype([('nodes', (int, 3)),('edges', (int,3))])
+                              dtype=np.dtype([('nodes', (int, 3)),
+                                              ('edges', (int, 3))])
                               )
         self.cells['nodes'] = new_cells_nodes
         self.cells['edges'] = new_cells_edges
-
         return
-    # --------------------------------------------------------------------------
+
     def compute_control_volumes(self, variant='voronoi'):
         '''Compute the control volumes of all nodes in the mesh.
         '''
@@ -273,12 +308,11 @@ class mesh2d(_base_mesh):
             self._compute_barycentric_volumes()
         else:
             raise ValueError('Unknown volume variant ''%s''.' % variant)
-
         return
-    # --------------------------------------------------------------------------
+
     def _compute_voronoi_volumes(self):
         num_nodes = len(self.node_coords)
-        self.control_volumes = np.zeros(num_nodes, dtype = float)
+        self.control_volumes = np.zeros(num_nodes, dtype=float)
 
         # compute cell circumcenters
         if self.cell_circumcenters is None:
@@ -311,31 +345,34 @@ class mesh2d(_base_mesh):
             # are coordinated such that in cell #i, the edge cellsEdges[i][k]
             # opposes cellsNodes[i][k].
             other0 = self.node_coords[self.cells['nodes'][cell0][edge_lid]] \
-                   - node
+                - node
             node_ids = self.edges['nodes'][edge_id]
             node_coords = self.node_coords[node_ids]
             edge_midpoint = 0.5 * (node_coords[0] + node_coords[1]) \
-                          - node
+                - node
             # Computing the triangle volume like this is called the shoelace
             # formula and can be interpreted as the z-component of the
             # cross-product of other0 and edge_midpoint.
             gauge = other0[0] * edge_midpoint[1] \
-                  - other0[1] * edge_midpoint[0]
+                - other0[1] * edge_midpoint[0]
 
             # Get the circumcenters of the adjacent cells.
             cc = self.cell_circumcenters[self.edges['cells'][edge_id]] \
-               - node
-            if len(cc) == 2: # interior edge
-                self.control_volumes[node_ids] += np.sign(gauge) \
-                                                * 0.5 * (cc[0][0] * cc[1][1] \
-                                                        -cc[0][1] * cc[1][0])
-            elif len(cc) == 1: # boundary edge
-                self.control_volumes[node_ids] += np.sign(gauge) \
-                                                * 0.5 * (cc[0][0] * edge_midpoint[1]
-                                                        -cc[0][1] * edge_midpoint[0])
+                - node
+            if len(cc) == 2:  # interior edge
+                self.control_volumes[node_ids] += \
+                    np.sign(gauge) * 0.5 * (cc[0][0] * cc[1][1]
+                                            - cc[0][1] * cc[1][0]
+                                            )
+            elif len(cc) == 1:  # boundary edge
+                self.control_volumes[node_ids] += \
+                    np.sign(gauge) * 0.5 * (cc[0][0] * edge_midpoint[1]
+                                            - cc[0][1] * edge_midpoint[0]
+                                            )
             else:
-                raise RuntimeError('An edge should have either 1 or two adjacent cells.')
-
+                raise RuntimeError('An edge should have either 1 '
+                                   'or two adjacent cells.'
+                                   )
         # Sanity checks.
         if self.cell_volumes is None:
             self.create_cell_volumes()
@@ -343,34 +380,32 @@ class mesh2d(_base_mesh):
         sum_cells = sum(self.cell_volumes)
         alpha = sum_cv - sum_cells
         if abs(alpha) > 1.0e-9:
-            msg = ('Sum of control volumes sum does not coincide with the sum of ' +
-                   'the cell volumes (|cv|-|cells| = %g - %g = %g.') \
-                  % (sum_cv, sum_cells, alpha)
+            msg = ('Sum of control volumes sum does not coincide with the '
+                   'sum of the cell volumes (|cv|-|cells| = %g - %g = %g.'
+                   ) % (sum_cv, sum_cells, alpha)
             raise RuntimeError(msg)
 
         if any(self.control_volumes < 0.0):
             msg = 'Not all control volumes are positive. This is due do ' \
                 + 'the triangulation not being Delaunay.'
             warnings.warn(msg)
-
         return
-    # --------------------------------------------------------------------------
+
     def _compute_barycentric_volumes(self):
-        '''Control volumes based on barycentric splitting.'''
+        '''Control volumes based on barycentric splitting.
+        '''
 
         # The barycentric midpoint "divides the triangle" into three
         # areas of equal volume. Hence, just assign one third of the
         # volumes to the corner points of each cell.
         if self.cell_volumes is None:
             self.create_cell_volumes()
-
         num_nodes = len(self.node_coords)
-        self.control_volumes = np.zeros(num_nodes, dtype = float)
+        self.control_volumes = np.zeros(num_nodes, dtype=float)
         for k, cell in enumerate(self.cells):
             self.control_volumes[cell['nodes']] += self.cell_volumes[k] / 3.0
-
         return
-    # --------------------------------------------------------------------------
+
     def compute_edge_normals(self):
         '''Compute the edge normals, pointing either in the direction of the
         cell with larger GID (for interior edges), or towards the outside of
@@ -390,7 +425,7 @@ class mesh2d(_base_mesh):
                 neighbor_cell_ids = self.edges['cells'][edge_id]
                 if cell_id == neighbor_cell_ids[0]:
                     edge_nodes = self.node_coords[self.edges['nodes'][edge_id]]
-                    edge = (edge_nodes[1] - edge_node[0])
+                    edge = (edge_nodes[1] - edge_nodes[0])
                     edge_normals[edge_id] = np.array([-edge[1], edge[0]])
                     edge_normals[edge_id] /= \
                         np.linalg.norm(edge_normals[edge_id])
@@ -398,12 +433,11 @@ class mesh2d(_base_mesh):
                     # Make sure the normal points in the outward direction.
                     other_node_id = self.cells['nodes'][cell_id][k]
                     other_node_coords = self.node_coords[other_node_id]
-                    if np.dot(edge_node[0]-other_node_coords,
+                    if np.dot(edge_nodes[0]-other_node_coords,
                               edge_normals[edge_id]) < 0.0:
                         edge_normals[edge_id] *= -1
-
         return edge_normals
-    # --------------------------------------------------------------------------
+
     def compute_gradient(self, u):
         '''Computes an approximation to the gradient \nabla u of a given
         scalar valued function u, defined in the node points.
@@ -415,7 +449,7 @@ class mesh2d(_base_mesh):
         '''
         num_nodes = len(self.node_coords)
         assert len(u) == num_nodes
-        gradient = np.zeros((num_nodes, 2), dtype = u.dtype)
+        gradient = np.zeros((num_nodes, 2), dtype=u.dtype)
 
         # Compute everything we need.
         if self.edges is None:
@@ -431,40 +465,50 @@ class mesh2d(_base_mesh):
         for edge_id, edge in enumerate(self.edges):
             if len(edge['cells']) == 1:
                 if edge['nodes'][0] not in boundary_matrices:
-                    boundary_matrices[edge['nodes'][0]] = np.zeros((2,2))
+                    boundary_matrices[edge['nodes'][0]] = np.zeros((2, 2))
                 if edge['nodes'][1] not in boundary_matrices:
-                    boundary_matrices[edge['nodes'][1]] = np.zeros((2,2))
+                    boundary_matrices[edge['nodes'][1]] = np.zeros((2, 2))
 
         for edge_id, edge in enumerate(self.edges):
             # Compute edge length.
             edge_coords = self.node_coords[edge['nodes'][1]] \
-                        - self.node_coords[edge['nodes'][0]]
+                - self.node_coords[edge['nodes'][0]]
 
             # Compute coedge length.
             if len(edge['cells']) == 1:
                 # Boundary edge.
                 edge_midpoint = 0.5 * (self.node_coords[edge['nodes'][0]]
-                                      +self.node_coords[edge['nodes'][1]])
+                                       + self.node_coords[edge['nodes'][1]]
+                                       )
                 coedge = self.cell_circumcenters[edge['cells'][0]] \
-                       - edge_midpoint
-                coedge_midpoint = 0.5 * (self.cell_circumcenters[edge['cells'][0]]
-                                        +edge_midpoint)
+                    - edge_midpoint
+                coedge_midpoint = \
+                    0.5 * (self.cell_circumcenters[edge['cells'][0]]
+                           + edge_midpoint
+                           )
             elif len(edge['cells']) == 2:
                 # Interior edge.
                 coedge = self.cell_circumcenters[edge['cells'][0]] \
-                       - self.cell_circumcenters[edge['cells'][1]]
-                coedge_midpoint = 0.5 * (self.cell_circumcenters[edge['cells'][0]]
-                                        +self.cell_circumcenters[edge['cells'][1]])
+                    - self.cell_circumcenters[edge['cells'][1]]
+                coedge_midpoint = \
+                    0.5 * (self.cell_circumcenters[edge['cells'][0]]
+                           + self.cell_circumcenters[edge['cells'][1]]
+                           )
             else:
-                raise RuntimeError('Edge needs to have either one or two neighbors.')
+                raise RuntimeError('Edge needs to have either '
+                                   'one or two neighbors.'
+                                   )
 
             # Compute the coefficient r for both contributions
-            coeffs = np.sqrt(np.dot(coedge, coedge) / np.dot(edge_coords, edge_coords)) \
-                   / self.control_volumes[edge['nodes']]
+            coeffs = np.sqrt(np.dot(coedge, coedge)
+                             / np.dot(edge_coords, edge_coords)
+                             ) / self.control_volumes[edge['nodes']]
 
             # Compute R*_{IJ} ((11) in [1]).
-            r0 = (coedge_midpoint - self.node_coords[edge['nodes'][0]])* coeffs[0]
-            r1 = (coedge_midpoint - self.node_coords[edge['nodes'][1]])* coeffs[1]
+            r0 = (coedge_midpoint - self.node_coords[edge['nodes'][0]]) \
+                * coeffs[0]
+            r1 = (coedge_midpoint - self.node_coords[edge['nodes'][1]]) \
+                * coeffs[1]
 
             diff = u[edge['nodes'][1]] - u[edge['nodes'][0]]
 
@@ -473,16 +517,18 @@ class mesh2d(_base_mesh):
 
             # Store the boundary correction matrices.
             if edge['nodes'][0] in boundary_matrices:
-                boundary_matrices[edge['nodes'][0]] += np.outer(r0, edge_coords)
+                boundary_matrices[edge['nodes'][0]] += \
+                    np.outer(r0, edge_coords)
             if edge['nodes'][1] in boundary_matrices:
-                boundary_matrices[edge['nodes'][1]] += np.outer(r1, -edge_coords)
+                boundary_matrices[edge['nodes'][1]] += \
+                    np.outer(r1, -edge_coords)
 
         # Apply corrections to the gradients on the boundary.
         for k, value in boundary_matrices.items():
             gradient[k] = np.linalg.solve(value, gradient[k])
 
         return gradient
-    # --------------------------------------------------------------------------
+
     def compute_curl(self, vector_field):
         '''Computes the curl of a vector field.
         While the vector field is point-based, the curl will be cell-based.
@@ -495,19 +541,20 @@ class mesh2d(_base_mesh):
         if self.edges is None:
             self.create_adjacent_entities()
 
-        curl = np.zeros((len(self.cells),3), dtype = vector_field.dtype)
+        curl = np.zeros((len(self.cells), 3), dtype=vector_field.dtype)
         for edge in self.edges:
             edge_coords = self.node_coords[edge['nodes'][1]] \
-                        - self.node_coords[edge['nodes'][0]]
+                - self.node_coords[edge['nodes'][0]]
             # Calculate A at the edge midpoint.
-            A = 0.5 * ( vector_field[edge['nodes'][0]]
-                      + vector_field[edge['nodes'][1]] )
-            print curl[edge['cells'],:]
-            print edge_coords
-            curl[edge['cells'],:] += edge_coords * np.dot(edge_coords, A)
+            A = 0.5 * (vector_field[edge['nodes'][0]]
+                       + vector_field[edge['nodes'][1]]
+                       )
+            print(curl[edge['cells'], :])
+            print(edge_coords)
+            curl[edge['cells'], :] += edge_coords * np.dot(edge_coords, A)
 
         return curl
-    # --------------------------------------------------------------------------
+
     def check_delaunay(self):
 
         if self.edges is None:
@@ -526,21 +573,19 @@ class mesh2d(_base_mesh):
 
             num_interior_edges += 1
 
-            # Each interior edge divides the domain into to half-planes.
-            # The Delaunay condition is fulfilled if and only if
-            # the circumcenters of the adjacent cells are in "the right order",
-            # i.e., line between the nodes of the cells which do not sit
-            # on the hyperplane have the same orientation as the line
-            # between the circumcenters.
+            # Each interior edge divides the domain into to half-planes.  The
+            # Delaunay condition is fulfilled if and only if the circumcenters
+            # of the adjacent cells are in "the right order", i.e., line
+            # between the nodes of the cells which do not sit on the hyperplane
+            # have the same orientation as the line between the circumcenters.
 
-            # The orientation of the coedge needs gauging.
-            # Do it in such as a way that the control volume contribution
-            # is positive if and only if the area of the triangle
-            # (node, other0, edge_midpoint) (in this order) is positive.
-            # Equivalently, the triangles (node, edge_midpoint, other1)
-            # or (node, other0, other1) could  be considered.
-            # other{0,1} refers to the the node opposing the edge in the
-            # adjacent cell {0,1}.
+            # The orientation of the coedge needs gauging.  Do it in such as a
+            # way that the control volume contribution is positive if and only
+            # if the area of the triangle (node, other0, edge_midpoint) (in
+            # this order) is positive.  Equivalently, the triangles (node,
+            # edge_midpoint, other1) or (node, other0, other1) could  be
+            # considered.  other{0,1} refers to the the node opposing the edge
+            # in the adjacent cell {0,1}.
             # Get the opposing node of the first adjacent cell.
             cell0 = self.edges['cells'][edge_id][0]
             # This nonzero construct is an ugly replacement for the nonexisting
@@ -562,10 +607,9 @@ class mesh2d(_base_mesh):
             # in the "same" direction.
             if np.dot(edge_midpoint-other0, cc[1]-cc[0]) < 0.0:
                 num_delaunay_violations += 1
-
         return num_delaunay_violations, num_interior_edges
-    # --------------------------------------------------------------------------
-    def show(self, show_covolumes = True, save_as=None):
+
+    def show(self, show_covolumes=True, save_as=None):
         '''Show the mesh using matplotlib.
 
         :param show_covolumes: If true, show all covolumes of the mesh, too.
@@ -599,22 +643,24 @@ class mesh2d(_base_mesh):
                 if len(ccs) == 2:
                     p = ccs.T
                 elif len(ccs) == 1:
-                    edge_midpoint = 0.5 * (self.node_coords[self.edges['nodes'][edge_id][0]]
-                                          +self.node_coords[self.edges['nodes'][edge_id][1]])
+                    edge_midpoint = \
+                        0.5 * (self.node_coords[self.edges['nodes'][edge_id][0]]
+                               + self.node_coords[self.edges['nodes'][edge_id][1]]
+                               )
                     p = np.c_[ccs[0], edge_midpoint]
                 else:
-                    raise RuntimeError('An edge has to have either 1 or 2 adjacent cells.')
-                ax.plot(p[0], p[1], color = covolume_col)
-
+                    raise RuntimeError('An edge has to have either 1 or 2 '
+                                       'adjacent cells.'
+                                       )
+                ax.plot(p[0], p[1], color=covolume_col)
         if save_as:
             import matplotlib2tikz
             matplotlib2tikz.save(save_as)
         else:
             plt.show()
-
         return
-    # --------------------------------------------------------------------------
-    def show_node(self, node_id, show_covolume = True):
+
+    def show_node(self, node_id, show_covolume=True):
         '''Plot the vicinity of a node and its covolume.
 
         :param node_id: Node ID of the node to be shown.
@@ -657,15 +703,20 @@ class mesh2d(_base_mesh):
                         q = np.c_[ccs[0], ccs[1], self.node_coords[node_id]]
                     elif len(ccs) == 1:
                         edge_midpoint = 0.5 * (self.node_coords[node_ids[0]]
-                                              +self.node_coords[node_ids[1]])
-                        p = np.c_[ccs[0], edge_midpoint]
-                        q = np.c_[ccs[0], edge_midpoint, self.node_coords[node_id]]
+                                               + self.node_coords[node_ids[1]]
+                                               )
+                        p = np.c_[ccs[0],
+                                  edge_midpoint
+                                  ]
+                        q = np.c_[ccs[0],
+                                  edge_midpoint,
+                                  self.node_coords[node_id]
+                                  ]
                     else:
-                        raise RuntimeError('An edge has to have either 1 or 2 adjacent cells.')
-                    ax.fill(q[0], q[1], color = covolume_area_col)
-                    ax.plot(p[0], p[1], color = covolume_boundary_col)
-
+                        raise RuntimeError('An edge has to have either 1 or 2 '
+                                           'adjacent cells.'
+                                           )
+                    ax.fill(q[0], q[1], color=covolume_area_col)
+                    ax.plot(p[0], p[1], color=covolume_boundary_col)
         plt.show()
         return
-    # --------------------------------------------------------------------------
-# ==============================================================================
